//
// defines.h
// Defines some aliases
//

#ifndef __DEFINES_H__
#define __DEFINES_H__


// Custom messages
#define WM_IF_DISPOSE    (WM_APP+6)
#define WM_GM_DISPOSE    (WM_APP+7)


// Sprite drawing modes
#define DRAW_PAINT                  0
#define DRAW_TRANS                  1


// Framebuffer modes
#define SCRMODE_NONE                0
#define SCRMODE_RAW_QVGA            1
#define SCRMODE_RAW_VGA             2
#define SCRMODE_GAPI_QVGA           3


// Screen type related to mouse input
#define SCRTYPE_UNKNOWN             0
#define SCRTYPE_VGA                 1
#define SCRTYPE_QVGA                2


// Interface constants
#define IFC_MAX_ELEMENTS_COUNT      8



// Game constants ////////////////////

// Maximum number of game elements
//   They divided by some areas (by z-order)
//   (I don't want to use variable-sized array of elements and z-order sorting before drawing)
// BGR0   - most far background - usually for stars and some special background elements
// BGR1, BGR2 - common background elements here
// FORE1 - common solid enemy objects (enemy ships)
// FORE2 - some upper objects, firing projectiles, sparks
// UP1   - blasts, fire and such upper scene coverage
// UP2   - most upper elements like fog, clouds, asteroid's dust and such
//
#define GM_MAX_ELEMENTS_COUNT       0x100
#define GM_AREA_BGR0                0x0F00 // it's a 16-elements [0x00 - 0x0F]
#define GM_AREA_BGR1                0x1F10 // also 16-elements [0x10 - 0x1F]
#define GM_AREA_BGR2                0x3F20 // ... guess
#define GM_AREA_FORE1               0x7F40 
#define GM_AREA_FORE2               0xBF80
#define GM_AREA_UP1                 0xEFC0
#define GM_AREA_UP2                 0xFFF0


// Some elements must be always in their place
#define GM_INDEX_RADIO              (GM_MAX_ELEMENTS_COUNT-1)
#define GM_INDEX_MAINSHIP           (GM_AREA_UP1 & 0x00FF)
#define GM_INDEX_FSTARS             0
#define GM_INDEX_SMOKE              ((GM_AREA_UP1 & 0x00FF)+1)
#define IF_INDEX_HUD                0


// Game modes
#define GM_MODE_UNKNOWN             0	// game mode - unknown
#define GM_MODE_NORMAL              1	// game mode - normal playing cycle
#define GM_MODE_MAINMENU            2   // main menu



// Sprites ////////////////////////////////////////////////

#define IMAGE_FONTS                 0
#define IMAGE_STARS                 1
#define IMAGE_MAINSHIP              2
#define IMAGE_ENGINE_FLAMES         3
#define IMAGE_FIREPRO_PLASMA        4
#define IMAGE_ENERGY_BAR            5
#define IMAGE_ASTEROIDS             6
#define IMAGE_STARDUST              7
#define IMAGE_ASTEROIDS_BGR1        8
#define IMAGE_ASTEROIDS_BGR2        9
#define IMAGE_ASTEROIDS_BGR3        10
#define IMAGE_ASTEROIDS_BGR4        11
#define IMAGE_ASTEROIDS_BGR5        12
#define IMAGE_ASTEROIDS_BGR6        13
#define IMAGE_ASTEROIDS_BGR7        14
#define IMAGE_ASTEROIDS_BGR8        15
#define IMAGE_RADIOFACES            16
#define IMAGE_RADIOTILES            17
#define IMAGE_BOOM_1                18
#define IMAGE_PARTICLES_1           19
#define IMAGE_PARTICLES_2           20
#define IMAGE_REDBIRD               21
#define IMAGE_ROCKET                22
#define IMAGE_SMOKE1                23
#define IMAGE_WASP                  24
#define IMAGE_ENGINE_FLAMES2        25
#define IMAGE_KRAIT                 26


// Sounds ///////////////////////////////////////////////////
#define SOUND_BOOM1                 0
#define SOUND_SHOOT1                1


// Script defines /////////////////////////////////////////

#define SCRIPT_NOFILE               0
#define SCRIPT_ENDFILE              1
#define SCRIPT_NOCOMMAND            2
#define SCRIPT_WAIT                 3
#define SCRIPT_INTERIOR             5
#define SCRIPT_ASTERSWARM           6
#define SCRIPT_GAMESPEED            7
#define SCRIPT_STARDUST             8
#define SCRIPT_RADIO                9
#define SCRIPT_MUSIC                10


// Game defs
#define FSTARS_MAX_COUNT            50  // Maximum amount of flying stars
#define PARTICLES_MAX_COUNT         10  // Maximum amount of particles



// Game elements types
#define CLASS_UNKNOWN               0


// Game elements subtypes
#define INTERIOR_ASTEROID_BGR       0
#define INTERIOR_STARDUST           1
#define FIREPRO_PLASMA              2
#define FIREPRO_ROCKET              3


// AI Modes
#define AI_MODE_NORMAL              0
#define AI_MODE_ZEROENERGY_PREPARE  1
#define AI_MODE_ZEROENERGY          2


// AutoCreator modes
#define AUTOCREATE_ASTERSWARM       1
#define AUTOCREATE_STARDUST         2


#endif // __DEFINES_H__
