//
// firepro.cpp
// Firing projectiles related classes
//

#include "stdafx.h"
#include "resource.h"



///////////////////////////////////////////////////////////////////
// Firepro class
/////////////////////////////////////////////////////////////////

TGame_firepro :: TGame_firepro ( TSurface * sf, int fx, int fy, int fsubtype ) : TGameElement( sf )
{
    x = fx;
    y = fy;
	esubtype = fsubtype;

    // per-subtype initialization
    switch (esubtype) {
      // plasma ball
      case FIREPRO_PLASMA:
          imgnum = IMAGE_FIREPRO_PLASMA;
          sprnum = 0;
          energy = 10;
          break;
      // rocket
      case FIREPRO_ROCKET:
          imgnum = IMAGE_ROCKET;
          sprnum = 0;
          energy = 20;
          break;
    }
}

void TGame_firepro :: Draw ( void )
{
    draw_sprite(shadow_buffer, (int)x, (int)y, crect_fullscreen, surface->images[imgnum], (int)sprnum, TRUE, is_highlight);
}

void TGame_firepro :: TimerTick ( void )
{
    if (destroy_this == TRUE) return;

    int dx = surface->images[imgnum]->dx[(int)sprnum];
    int dy = surface->images[imgnum]->dy[(int)sprnum];
    int ih;

    switch (esubtype) {

      // plasma ball
      case FIREPRO_PLASMA:

        switch (ai_mode) {
          // normal flying 
          case AI_MODE_NORMAL:
            // check for hitting obstacles or enemy ship
            if (surface->CollisionCheck((int)x, (int)y, dx, dy, &ih, NULL, NULL)) {
              surface->game_elements[ih]->energy -= energy;
              surface->game_elements[ih]->is_highlight = TRUE;
              surface->Score += energy;
              if (surface->game_elements[ih]->energy <= 0) surface->game_elements[ih]->ai_mode = AI_MODE_ZEROENERGY_PREPARE;
              ai_mode = AI_MODE_ZEROENERGY_PREPARE;
            } else {
              y -= 10; // projectiles will not be affected by global game speed
              // if we are out of sight - dispose projectile
              if ((y+(dy >> 1)) < 0) destroy_this = TRUE;
            }
            break;

          // preparing to zero-energy (self-destruction)
          case AI_MODE_ZEROENERGY_PREPARE:
            sprnum  = 1;
            ai_mode = AI_MODE_ZEROENERGY;
            break;

          // self-destruct mode
          case AI_MODE_ZEROENERGY:
            if ((int)sprnum < surface->images[imgnum]->count) sprnum += 0.5; else destroy_this = TRUE;
            break;
        }
        break;


      // pulse rocket
      case FIREPRO_ROCKET:

        switch (ai_mode) {
          // normal flying 
          case AI_MODE_NORMAL:
            // check for hitting obstacles or enemy ship
            if (surface->CollisionCheck((int)x, (int)y, dx, dy, &ih, NULL, NULL)) {
              surface->game_elements[ih]->energy -= energy;
              surface->game_elements[ih]->is_highlight = TRUE;
              surface->Score += energy;
              if (surface->game_elements[ih]->energy <= 0) surface->game_elements[ih]->ai_mode = AI_MODE_ZEROENERGY_PREPARE;
              ai_mode = AI_MODE_ZEROENERGY_PREPARE;
            } else {
              // add some smoke
              ((TGame_smoke *)(surface->game_elements[GM_INDEX_SMOKE]))->AddSmoke((int)x, (int)y, IMAGE_SMOKE1);
              y -= 20; // projectiles will not be affected by global game speed
              // if we are out of sight - dispose projectile
              if ((y+(dy >> 1)) < 0) destroy_this = TRUE;
            }
            break;

          // preparing to zero-energy (self-destruction)
          case AI_MODE_ZEROENERGY_PREPARE:
          case AI_MODE_ZEROENERGY:
            destroy_this = TRUE;
            break;
        }
        break;
    
    } // switch esubtype
}

