//
// gfxdraw.cpp
// Stuff related to drawing sprites and PDA's framebuffer operations
//
#include "stdafx.h"


//////////////////////////////////////////////////////////////////////

    // Raw framebuffer structure
    RawFrameBufferInfo  screen_rfbi;

    // Screen dimensions for our shadow framebuffer (not actual ones)
    int                 screen_dx;
    int                 screen_dy;
    int                 screen_size;
    int                 screen_mode;

    // Brightness value (0 - normal .. 6 - dark)
    int                 screen_brightness;
    int                 screen_brightness_dir;

	// Shadow framebuffer
	LPWORD				shadow_buffer;

    // Clipping regions
    RECT                crect_fullscreen;

    // 454 -> 565 color scheme transformation table
    LPWORD              ctrans_454_565;

    // first transparent color conversion table (applied to sprite colors)
    LPWORD              alpha_table_src;
    // second transparent color conversion table (applied to shadow buffer colors)
    LPWORD              alpha_table_dst;

    // 454 colors conversion table - highlight (r,g,b = r*2,g*2,b*2) (for non-premultiplied colors)
    LPWORD              hlight_table_normal;
    // 454 colors conversion table - highlight (r,g,b = r*2,g*2,b*2) (for premultiplied colors)
    LPWORD              hlight_table_premul;



// assembler routines from asmdraw.asm
//
extern "C" int asm_draw_block    ( LPWORD fbuf, LPWORD sprbuf, DWORD dxdy, DWORD sdx);
extern "C" int asm_draw_tblock   ( LPWORD fbuf, LPWORD sprbuf, DWORD dxdy, DWORD sdx, LPWORD col_trans);
extern "C" int asm_draw_trans    ( LPWORD fbuf, LPWORD sprbuf, DWORD dxdy, DWORD sdx, LPWORD dst_trans);
extern "C" int asm_upscale_ffb   ( LPWORD dst, LPWORD src, LPWORD colortrans );
extern "C" int asm_noscale_ffb   ( LPWORD dst, LPWORD src, LPWORD colortrans );

//////////////////////////////////////////////////////////////////////

// Draws sprite to buffer
//
void draw_sprite( LPWORD fbuf, int x, int y, RECT clip_rect, TImage * img, int sprnum, BOOL centered, BOOL highlight )
{
    int     __x, __y;
    int     sx, sy, sdx, sdy;
	int		ddx, ddy;
    LPWORD  spraddr, bufaddr;

    if (img == NULL) return;
    if ((sprnum < 0) || (sprnum >= img->count)) return;

	sdx = img->dx[sprnum]; 
	sdy = img->dy[sprnum]; 

	// if centered - adjust coordinates
	if (centered == TRUE) {
	  x = x - (sdx>>1);
	  y = y - (sdy>>1);
	}

	// testing if we really need draw sprite or maybe it's off-screen
    __x = x; if ((__x >= clip_rect.right) || ((__x + sdx) <= clip_rect.left)) return;
    __y = y; if ((__y >= clip_rect.bottom) || ((__y + sdy) <= clip_rect.top)) return;

	// adjust "starting point" and sizes of our drawing rectangle
    sx = 0; if (__x<clip_rect.left) { sx = (clip_rect.left-__x); __x = clip_rect.left; }
    sy = 0; if (__y<clip_rect.top) { sy = (clip_rect.top-__y); __y = clip_rect.top; }
	ddx = sdx - sx; if ((clip_rect.right - __x) < ddx) ddx = (clip_rect.right - __x);
	ddy = sdy - sy; if ((clip_rect.bottom - __y) < ddy) ddy = (clip_rect.bottom - __y);

    spraddr = &((img->data[sprnum])[sy*sdx+sx]);
    bufaddr = &(fbuf[__y*screen_dx+__x]);

	switch (img->isalpha[sprnum]) {
	  case 0: // draws block without using aplha channel 
           if (highlight) asm_draw_tblock(bufaddr, spraddr, (DWORD)((ddy & 0xFFFF) + (ddx << 16)), (DWORD)sdx, (LPWORD)hlight_table_normal);
                     else asm_draw_block(bufaddr, spraddr, (DWORD)((ddy & 0xFFFF) + (ddx << 16)), (DWORD)sdx);
		   break;
	  case 1: // draws block with alpha channel 
		   asm_draw_trans(bufaddr, spraddr, (DWORD)((ddy & 0xFFFF) + (ddx << 16)), (DWORD)sdx, (LPWORD)alpha_table_dst);
		   break;
	}
}

// Draws text block to buffer
//
void draw_string( LPWORD fbuf, int x, int y, RECT clip_rect, LPCTSTR s, TImage * img, int sprnum, int fdx, int fdy )
{
	int	  i, l, base_x, sx, sy;
	TCHAR c;
    LPWORD  spraddr, bufaddr;

    // basic checks
    if (img == NULL) return;
    if (sprnum >= img->count) return;
	if (x >= clip_rect.right)  return;
	if (y >= clip_rect.bottom) return;
    l = wcslen(s); if (l==0) return;

	i = 0;
    base_x = x;

	while (i < l) {
      c = s[i];
      // newline character
      if (c == '\\') {
        x = base_x;
        y += (fdy+1);
        i++;
        continue;
      }
      // draws character (if we are in clipping rect)
      if ((x>=clip_rect.left) && ((x+fdx)<clip_rect.right) && (y>=clip_rect.top) && ((y+fdy)<clip_rect.bottom)) {
        if ((c >= '!') && (c <= 0x7F)) {
          sy = ((int)(c - '!') >> 3) * fdy;
          sx = ((int)(c - '!') & 0x7) * fdx;
          if ((sy+fdy) <= (img->dy[sprnum])) {
	        spraddr = &((img->data[sprnum])[sy*img->dx[sprnum] + sx]);
	        bufaddr = &(fbuf[y*screen_dx+x]);
            switch (img->isalpha[sprnum]) {
              case 0:
                  asm_draw_block(bufaddr, spraddr, (DWORD)((fdy & 0xFFFF) + (fdx << 16)), (DWORD)(img->dx[sprnum]));
                  break;
              case 1:
                  asm_draw_trans(bufaddr, spraddr, (DWORD)((fdy & 0xFFFF) + (fdx << 16)), (DWORD)(img->dx[sprnum]), (LPWORD)alpha_table_dst);
                  break;
            }
          }
		}
      }
	  i++;
	  x += fdx;
	}
}

// Draws horizontal line
//
void draw_hline ( LPWORD fbuf, int x, int y, int len, WORD color, int alpha, RECT clip_rect )
{
    LPWORD  bufaddr;
    WORD    __alpha, __color;
    int     __x, __y, __len;
    int     i;

    if ((x >= clip_rect.right) || ((x+len) <= clip_rect.left)) return;
    if ((y < clip_rect.top) || (y >= clip_rect.bottom)) return;

    __x   = x;
    __y   = y;
    __len = len;
    if (__x < clip_rect.left) { __len = __len - (clip_rect.left-__x); __x = clip_rect.left; }
    if ((__x+__len) > clip_rect.right) __len = clip_rect.right - __x;

    bufaddr = &(fbuf[__y*screen_dx+__x]);
    __alpha = (alpha & 7) << 13;
    __color = alpha_table_src[__alpha | (color & 0x1FFF)];
    for (i=0; i<__len; i++)  bufaddr[i] = __color + alpha_table_dst[__alpha | (bufaddr[i] & 0x1FFF)];
}

// Draws vertical line
//
void draw_vline ( LPWORD fbuf, int x, int y, int len, WORD color, int alpha, RECT clip_rect )
{
    LPWORD  bufaddr;
    WORD    __alpha, __color;
    int     __x, __y, __len;
    int     i,j;

    if ((y >= clip_rect.bottom) || ((y+len) <= clip_rect.top)) return;
    if ((x < clip_rect.left) || (x >= clip_rect.right)) return;

    __x   = x;
    __y   = y;
    __len = len;
    if (__y < clip_rect.top) { __len = __len - (clip_rect.top-__y); __y = clip_rect.top; }
    if ((__y+__len) > clip_rect.bottom) __len = clip_rect.bottom - __y;

    bufaddr = &(fbuf[__y*screen_dx+__x]);
    __alpha = (alpha & 7) << 13;
    __color = alpha_table_src[__alpha | (color & 0x1FFF)];
    for (i=0,j=0; i<__len; i++)  { 
        bufaddr[j] = __color + alpha_table_dst[__alpha | (bufaddr[j] & 0x1FFF)];
        j += screen_dx;
    }
}

// Creates conversion table 454 --> 565 colorspace
//
BOOL create_454_565_table ( void )
{
    const int larr[] = {1,2,4,6,8,10,12};
    int   i;
    DWORD c;
    WORD  r1, g1, b1;

    ctrans_454_565 = (LPWORD) malloc( 0x2000 * 7 * sizeof(WORD));
    if (ctrans_454_565 == NULL) return FALSE;

    // 7 light levels: 0-normal..6-most darkened
    for (i=0; i<7; i++) {
        for (c=0; c<0x2000; c++) {
            r1 = (WORD)(c & 0x1E00) >> 9; // initial color values
            g1 = (WORD)(c & 0x01F0) >> 4; //
            b1 = (WORD)(c & 0x000F);      // 

            r1 = (WORD)(((double)r1) / ((double)larr[i])); // adjusted with light color values 
            g1 = (WORD)(((double)g1) / ((double)larr[i])); //
            b1 = (WORD)(((double)b1) / ((double)larr[i])); //

            r1 = (r1 & 0x0F) << 1; // resulting 565 color values
            g1 = (g1 & 0x1F) << 1; //
            b1 = (b1 & 0x0F) << 1; //

            ctrans_454_565[i*0x2000 + c] = (r1<<11) + (g1<<5) + b1;
        }
    }

    return TRUE;
}

// Creates alpha-blending tables
//
BOOL create_alpha_tables ( void )
{
    DWORD c;
    WORD av, cv, cr, cg, cb, cr1, cg1, cb1, cr2, cg2, cb2;

    alpha_table_src = (LPWORD) malloc ( 0x10000 * sizeof(WORD));
    if (alpha_table_src == NULL) return FALSE;
    alpha_table_dst = (LPWORD) malloc ( 0x10000 * sizeof(WORD));
    if (alpha_table_dst == NULL) return FALSE;

    for (c=0; c<=0xFFFF; c++) {
      av = (WORD)(c & 0xE000) >> 13; // alpha value 0..7
      cv = (WORD)(c & 0x1FFF);       // color value

      cr = (cv & 0x1E00) >> 9; // 4-bits
      cg = (cv & 0x01F0) >> 4; // 5-bits
      cb = (cv & 0x000F);      // 4-bits

      // first transformed value - will be applied to sprite colors
      cr1 = (WORD)(round((double)cr * (1.0 - ((double)av / 7.0))));
      cg1 = (WORD)(round((double)cg * (1.0 - ((double)av / 7.0))));
      cb1 = (WORD)(round((double)cb * (1.0 - ((double)av / 7.0))));

      alpha_table_src[c] = (cr1 << 9) + (cg1 << 4) + cb1;

      // second transformed value - will be applied to framebuffer colors
      cr2 = (WORD)(round((double)cr * ((double)av / 7.0)));
      cg2 = (WORD)(round((double)cg * ((double)av / 7.0)));
      cb2 = (WORD)(round((double)cb * ((double)av / 7.0)));
      // adjust second colors to not exceed their maximum
      while ((cr1+cr2) > 0x0F) cr2--;
      while ((cg1+cg2) > 0x1F) cg2--;
      while ((cb1+cb2) > 0x0F) cb2--;

      alpha_table_dst[c] = (cr2 << 9) + (cg2 << 4) + cb2;
    }

    return TRUE;
}

// Creates highlight tables
//
BOOL create_hlight_tables ( void )
{
    DWORD c;
    WORD  r1, g1, b1;

    hlight_table_normal = (LPWORD) malloc ( 0x2000 * sizeof(WORD));
    if (hlight_table_normal == NULL) return FALSE;

    for (c=0; c<0x2000; c++) {
      r1 = (WORD)(c & 0x1E00) >> 9; // 4-bits
      g1 = (WORD)(c & 0x01F0) >> 4; // 5-bits
      b1 = (WORD)(c & 0x000F);      // 4-bits

      r1 = r1 << 1; if (r1 > 0x0F) r1 = 0x0F;
      g1 = g1 << 1; if (g1 > 0x1F) g1 = 0x1F;
      b1 = b1 << 1; if (b1 > 0x0F) b1 = 0x0F;

      hlight_table_normal[c] = (r1 << 9) + (g1 << 4) + (b1);
    }

    return TRUE;
}

////////////////////////////////////////////////////////////////////////////////////////

TImage :: TImage ( LPCTSTR fname, BOOL load_compressed )
{
    HANDLE  finp;
    DWORD   dw, l;
    int     i;
    LPBYTE  comp_data;
    DWORD   comp_size;

    count = 0;
    dx = NULL;
    dy = NULL;
    isalpha = NULL;
    data = NULL;
    is_disposable = FALSE;

    finp = CreateFile (fname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (!finp) return;

    ReadFile(finp, &count, 2, &dw, NULL);

    dx       = (LPWORD) malloc(count * sizeof(WORD));
    dy       = (LPWORD) malloc(count * sizeof(WORD));
    isalpha  = (LPWORD) malloc(count * sizeof(WORD));    
    data     = (LPWORD *) malloc(count * sizeof(LPWORD));
    datasize = (LPDWORD) malloc(count * sizeof(DWORD));

    for (i=0; i<count; i++) {

      ReadFile(finp, &(dx[i]), 2, &dw, NULL); l = dx[i];
      ReadFile(finp, &(dy[i]), 2, &dw, NULL); l *= dy[i];
      ReadFile(finp, &(isalpha[i]), 2, &dw, NULL);
      l *= sizeof(WORD);
      // compressed data size
      ReadFile(finp, &comp_size, 4, &dw, NULL);

      // reading compressed data
      comp_data = (LPBYTE) malloc(comp_size);
      ReadFile(finp, comp_data, comp_size, &dw, NULL);

      if (load_compressed == FALSE) {
        // decompress it
        data[i] = (LPWORD) malloc (l);
        datasize[i] = l;
        uncompress((LPBYTE)(data[i]), &l, comp_data, comp_size);
      } else {
        // do not decompress
        data[i] = (LPWORD) malloc ((comp_size&0xFFFFFFFE)+2);
        datasize[i] = comp_size;
        memcpy((LPBYTE)(data[i]), (LPBYTE)comp_data, comp_size);
      }

      free(comp_data);
    }

	CloseHandle(finp);
}

TImage :: ~TImage ()
{
    int i;
    if (count == 0) return;
	free(dx);
	free(dy);
	free(isalpha);
	for (i=0; i<count; i++) free(data[i]);
}

// Loads all image files (*.454) from folder (as uncompressed)
//
void LoadImagesFromFolder ( TImage ** images, LPCTSTR sfolder, BOOL __disposable )
{
    TCHAR               s[2048];
    LPTSTR              lps;
    HANDLE              ff;
    WIN32_FIND_DATA     fd;
    int                 i;

    wsprintf(s, L"%s%s*.454", g_sCurrentDir, sfolder);
	ff = FindFirstFile(s, &fd);
	if (ff != INVALID_HANDLE_VALUE) {
      wsprintf(s, L"%s", fd.cFileName);
      lps = wcschr(s, '_');
      if (lps != NULL) {
        *lps = 0;
        i = wcstol(s, NULL, 10);
        if ((i>=0) && (i<MAX_IMAGES_COUNT) && (images[i] == NULL)) {
          wsprintf(s, L"%s%s%s", g_sCurrentDir, sfolder, fd.cFileName);
          images[i] = new TImage(s, FALSE/* decompress */);
          images[i]->is_disposable = __disposable;
        }
      }
      while (FindNextFile(ff, &fd)) {
        wsprintf(s, L"%s", fd.cFileName);
        lps = wcschr(s, '_');
        if (lps != NULL) {
          *lps = 0;
          i = wcstol(s, NULL, 10);
          if ((i>=0) && (i<MAX_IMAGES_COUNT) && (images[i] == NULL)) {
            wsprintf(s, L"%s%s%s", g_sCurrentDir, sfolder, fd.cFileName);
            images[i] = new TImage(s, FALSE/* decompress */);
            images[i]->is_disposable = __disposable;
          }
        }
      }
	  FindClose(ff);
	}
}



// Try to init framebuffer mode - RAW or GAPI
//
BOOL InitGfxMode ( void )
{
    HDC hdc;
    GXDisplayProperties dispprop;

    screen_mode = SCRMODE_NONE;

    screen_brightness = 0;
    screen_brightness_dir = 0;

    // Trying to init raw framebuffer mode first - for WM2003SE or WM5 devices
    hdc = GetDC(NULL);
    ExtEscape(hdc, GETRAWFRAMEBUFFER, 0, NULL, sizeof(RawFrameBufferInfo), (char *) &screen_rfbi);
    ReleaseDC(NULL, hdc);

    if ((screen_rfbi.wBPP == 16) && (screen_rfbi.wFormat == FORMAT_565))
	{
        // Ok, we are now in 16bpp 565 mode, it's good, test resolution then
		if ((screen_rfbi.cxPixels == 240) && (screen_rfbi.cyPixels == 320)) { screen_mode = SCRMODE_RAW_QVGA; return TRUE; }
		if ((screen_rfbi.cxPixels == 480) && (screen_rfbi.cyPixels == 640)) { screen_mode = SCRMODE_RAW_VGA; return TRUE; }
    }

    // Well... if failed - then try to init 240x320 16bpp GAPI mode
    if (GXOpenDisplay(g_hMainWindow, GX_FULLSCREEN) == 0) {
      AddToGlobalErrors(L"GAPI: Unable to init display.");
      return FALSE;
    }
    dispprop = GXGetDisplayProperties();
    
    if ((dispprop.cBPP != 16) || (dispprop.cxWidth != 240) || (dispprop.cyHeight != 320))
    {
      GXCloseDisplay();
      AddToGlobalErrors(L"GAPI: Unable to init 240x320 16bpp mode.");
      return FALSE;
    }

    // Ok, at least able to init GAPI mode
    screen_mode = SCRMODE_GAPI_QVGA;
    return TRUE;
}

// Initialize graphics
//
BOOL InitGfx ( void )
{
	if (!InitGfxMode()) return FALSE;

    // Draw 'Wait...' picture 
    if (screen_mode == SCRMODE_RAW_QVGA) {
      memset((LPBYTE)screen_rfbi.pFramePointer, 0, 320*240*sizeof(WORD));
      memcpy((LPBYTE)(&(((LPWORD)screen_rfbi.pFramePointer)[(160-11)*240])), init_gfx_wait_240, 240*22*sizeof(WORD));
    }
    if (screen_mode == SCRMODE_RAW_VGA) {
      memset((LPBYTE)screen_rfbi.pFramePointer, 0, 640*480*sizeof(WORD));
      memcpy((LPBYTE)(&(((LPWORD)screen_rfbi.pFramePointer)[(320-22)*480])), init_gfx_wait_480, 480*44*sizeof(WORD));
    }
    if (screen_mode == SCRMODE_GAPI_QVGA) {
      LPWORD framebuffer = (LPWORD) GXBeginDraw();
      memset((LPBYTE)framebuffer, 0, 320*240*sizeof(WORD));
      memcpy((LPBYTE)(&(framebuffer[(160-11)*240])), (LPBYTE)init_gfx_wait_240, 240*22*sizeof(WORD));
      GXEndDraw();
    }

	// framebuffer size is SAME for vga and qvga - it's only for internal framebuffer, not for actual display
    screen_dx = 240;
    screen_dy = 320;
    screen_size = screen_dx * screen_dy;

    // init default fullscreen clipping region
    // (!!!) left and top sides are INCLUSIVE, right and bottom are NOT (!!!)
    crect_fullscreen.left = 0;
    crect_fullscreen.right = 240;
    crect_fullscreen.top = 0;
    crect_fullscreen.bottom = 320;

	// allocate shadow framebuffer
	shadow_buffer = (LPWORD) malloc (screen_size * sizeof(WORD));
	if (shadow_buffer == NULL) return FALSE;

    // create color-transformation tables
    if (create_454_565_table() == FALSE) {
      AddToGlobalErrors(L"Unable to create 454->565 transformation table (memory low?)");
      return FALSE;
    }
    if (create_alpha_tables() == FALSE) {
      AddToGlobalErrors(L"Unable to create alpha-blending tables (memory low?)");
      return FALSE;
    }
    if (create_hlight_tables() == FALSE) {
      AddToGlobalErrors(L"Unable to create highlight tables (memory low?)");
      return FALSE;
    }

    // This must draw blank black screen
    screen_brightness = 7;
    DrawShadowBuffer();

	return TRUE;
}

// Gfx cleanup
//
void DeinitGfx ( void )
{
	// free GAPI resources if needed
    if (screen_mode == SCRMODE_GAPI_QVGA) GXCloseDisplay();

	// free our framebuffer
	free(shadow_buffer);

    // free color-transformation tables
    free(ctrans_454_565);
    free(alpha_table_src);
    free(alpha_table_dst);
    free(hlight_table_normal);
    free(hlight_table_premul);
}

// Draw our QVGA buffer on screen
//
void DrawShadowBuffer ( void )
{
    if (screen_brightness < 0) screen_brightness = 0;
    if (screen_brightness > 7) screen_brightness = 7;
    
	// plain copy in QVGA RAW mode
    if (screen_mode == SCRMODE_RAW_QVGA) {
      if (screen_brightness < 7)
        asm_noscale_ffb((LPWORD)screen_rfbi.pFramePointer, shadow_buffer, (LPWORD)(&(ctrans_454_565[screen_brightness*0x2000])));
      else
        memset((LPBYTE)screen_rfbi.pFramePointer, 0, 320*240*sizeof(WORD));
    }

	// scaled copy in VGA RAW mode
    if (screen_mode == SCRMODE_RAW_VGA) {
      if (screen_brightness < 7)
        asm_upscale_ffb((LPWORD)screen_rfbi.pFramePointer, shadow_buffer, (LPWORD)(&(ctrans_454_565[screen_brightness*0x2000])));
      else
        memset((LPBYTE)screen_rfbi.pFramePointer, 0, 640*480*sizeof(WORD));
    }

	// also plain copy in QVGA GAPI mode
    if (screen_mode == SCRMODE_GAPI_QVGA) {
      LPWORD framebuffer = (LPWORD) GXBeginDraw();
      if (screen_brightness < 7)
        asm_noscale_ffb(framebuffer, shadow_buffer, (LPWORD)(&(ctrans_454_565[screen_brightness*0x2000])));
      else
        memset((LPBYTE)framebuffer, 0, 320*240*sizeof(WORD));
      GXEndDraw();
    }

    if (screen_brightness_dir == 1) screen_brightness++;
    if (screen_brightness_dir == -1) screen_brightness--;
}
