//
// interior.cpp
// Some classes related to 'interior' type objects
//

#include "stdafx.h"
#include "resource.h"


//////////////////////////////////////////////////////////////////////////
// "Floating" stars class 
////////////////////////////////////////////////////////////////////////

TGame_fstars :: TGame_fstars ( TSurface * sf, int num ) : TGameElement( sf )
{
    int i;

    imgnum = IMAGE_STARS;
    sprnum = 0;

    // stars count must be 0..FSTARS_MAX_COUNT
    count = num;
    if (count < 0) count = 0; 
      else if (count > FSTARS_MAX_COUNT) count = FSTARS_MAX_COUNT;

    // place stars randomly on screen
    for (i=0; i<count; i++) {
      fsprite[i] = MyRandom(0, 3);
      fx[i] = MyRandom(1, screen_dx-1);
      fy[i] = (double)MyRandom(-40, screen_dy+40);
      fspeed[i] = MyRandom(1, 4) / 2.0;
    }
}

void TGame_fstars :: Draw ( void )
{
    int i;

    for (i=0; i<count; i++)
      draw_sprite(shadow_buffer, (int)fx[i], (int)fy[i], crect_fullscreen, surface->images[imgnum], fsprite[i]+(int)sprnum, TRUE, FALSE);
}

void TGame_fstars :: TimerTick ( void )
{
    int i;

    sprnum = 0;
    //if (GAME_SPEED_KOEFF < 0.8 ) sprnum = 4;
    if (GAME_SPEED_KOEFF < 0.3 ) sprnum = 4;

    // increase star's Y by it's speed factor
    for (i=0; i<count; i++) {
      fy[i] += (fspeed[i] / GAME_SPEED_KOEFF);
      // if it's fly out of screen
      if (fy[i] > (screen_dy+40)) {
        fy[i] = -40;
        fx[i] = MyRandom(1, screen_dx-1);
        fspeed[i] = MyRandom(1, 4) / 2.0;
      }
    }
}

////////////////////////////////////////////////////////////////
// Particles class 
//////////////////////////////////////////////////////////////

TGame_particles :: TGame_particles ( TSurface * sf, double __x, double __y, int num, int __imgnum ) : TGameElement( sf )
{
    int i;
    if (num > PARTICLES_MAX_COUNT) num = PARTICLES_MAX_COUNT;
    fcount = num;
    fcenter_x = __x;
    fcenter_y = __y;
    for (i=0; i<fcount; i++) {
      fx[i] = __x + MyRandom(-1,1);
      fy[i] = __y + MyRandom(-1,1);
      fimage[i] = __imgnum;
      fsprite[i] = 0;
      fdx[i] = MyRandom(-15,15);
      fdy[i] = MyRandom(-15,15);
    }
}

void TGame_particles :: Draw ( void )
{
    int i;
    // must draw trail from tail to head
    for (i=0; i<fcount; i++) {
        if (fimage[i] >= 0) {
            draw_sprite(shadow_buffer, (int)fx[i], (int)fy[i], crect_fullscreen, surface->images[fimage[i]], (int)fsprite[i], TRUE, FALSE);
        }
    }
}

void TGame_particles :: TimerTick ( void )
{
    int i;
    for (i=0; i<fcount; i++) if (fimage[i] >= 0) {
      fx[i] = fx[i] + fdx[i];
      fdx[i] = fdx[i] / 1.1;
      fy[i] = fy[i] + fdy[i];
      fdy[i] = fdy[i] / 1.1;
      fsprite[i] += 0.6;
      if ((int)fsprite[i] >= surface->images[fimage[i]]->count) fimage[i] = -1;
    }
    for (i=0; i<fcount; i++) if (fimage[i] >= 0) return;
    destroy_this = TRUE;
}


///////////////////////////////////////////////////////////////////
// Interior class
/////////////////////////////////////////////////////////////////

TGame_interior :: TGame_interior ( TSurface * sf, int fx, int fy, int fsubtype, int fimgnum ) : TGameElement( sf )
{
    x = fx;
    y = fy;
    esubtype = fsubtype;
    imgnum   = fimgnum;
    sprnum   = 0;
}

void TGame_interior :: Draw ( void )
{
    draw_sprite(shadow_buffer, (int)x, (int)y, crect_fullscreen, surface->images[imgnum], (int)sprnum, FALSE, is_highlight);
}

void TGame_interior :: TimerTick ( void )
{
    y += (speed / GAME_SPEED_KOEFF);
    // if we are out of sight - dispose
    if (y > screen_dy) destroy_this = TRUE;
    IncSprNum(0.5);
}

