//
// mainship.cpp
// Our main ships classes
//

#include "stdafx.h"
#include "resource.h"

////////////////////////////////////////////////////////////////
// Main ship class
//////////////////////////////////////////////////////////////

TGame_mainship :: TGame_mainship ( TSurface * sf ) : TGameElement( sf )
{
    // initial ship position
    x = (screen_dx >> 1);
    y = (screen_dy - 100);
    // energy
    energy = 100;
    // inertion moment and weight
	inert_dx = 0;
	inert_dy = 0;
    weight   = 4;
    // subtype
    esubtype = SUBTYPE_MAINSHIP_DARKSTAR;
    // sprites
    imgnum = IMAGE_MAINSHIP; // default, but when drawing ship - will be changed according to <esubtype>
    sprnum = 2.5;            // mainship has a 5 angles for viewing it - this is a center
    // engine flames sprite
    engine_sprnum = 0;
}

void TGame_mainship :: Draw ( void )
{
    int i;

    // change ship image according to current type of main ship
    switch (esubtype) {
      case SUBTYPE_MAINSHIP_KRAIT:    imgnum = IMAGE_KRAIT;    break;
      case SUBTYPE_MAINSHIP_DARKSTAR: imgnum = IMAGE_MAINSHIP; break;
    }

    // ship's hull
    draw_sprite(shadow_buffer, (int)x, (int)y, crect_fullscreen, surface->images[imgnum], (int)sprnum, TRUE, is_highlight);

    // engine flames
    if (engine_sprnum >= 0) {
      switch (esubtype) {
        case SUBTYPE_MAINSHIP_DARKSTAR:
          i = abs((int)sprnum-2);
          draw_sprite(shadow_buffer, (int)x-16+i, (int)y+19, crect_fullscreen, surface->images[IMAGE_ENGINE_FLAMES], (int)engine_sprnum, FALSE, FALSE);
          draw_sprite(shadow_buffer, (int)x+8-i, (int)y+19, crect_fullscreen, surface->images[IMAGE_ENGINE_FLAMES], (int)engine_sprnum, FALSE, FALSE);
          break;
        case SUBTYPE_MAINSHIP_KRAIT:
          draw_sprite(shadow_buffer, (int)x-3, (int)y+17, crect_fullscreen, surface->images[IMAGE_ENGINE_FLAMES2], (int)engine_sprnum, FALSE, FALSE);
      }
    }
}

void TGame_mainship :: TimerTick ( void )
{
    int i, dx, dy, idx, idy, ih;

    // no engine flame by default
    engine_sprnum += 0.5;
    if ((int)engine_sprnum > 1) engine_sprnum = 0;

    // reset ship rotation
    sprnum += ((2.5 - sprnum) / 4.0);

    // moving ship to stylus coordinates
    if (surface->stylus_down == 1) {
      dx = (int) floor(abs(surface->stylus_last_x - (int)x) / 3);
      dy = (int) floor(abs(surface->stylus_last_y - (int)y) / 3);
      if (dx > 20) dx = 20;
      if (dy > 20) dy = 20;
      if (surface->stylus_last_x > x) {
        x += dx;
        if (dx > 2) sprnum += (dx/10.0);
      }
      if (surface->stylus_last_x < x) {
        x -= dx;
        if (dx > 2) sprnum -= (dx/10.0);
      }
      if (surface->stylus_last_y > y) y += dy;
      if (surface->stylus_last_y < y) {
        y -= dy;
      }
    }
    if ((int)sprnum < 0) sprnum = 0;
    if ((int)sprnum > 4) sprnum = 4;

	AdjustInertion();

	if (x > (screen_dx-20)) x = screen_dx-20; else if (x < 20) x = 20;
	if (y > (screen_dy-20)) y = screen_dy-20; else if (y < 20) y = 20;

	// check collision with other solid game objects
    if (surface->CollisionCheck((int)x, (int)y, 
		                        surface->images[imgnum]->dx[(int)sprnum], 
								surface->images[imgnum]->dy[(int)sprnum], 
								&ih, &idx, &idy)) 
	{
	   inert_dx = idx / weight; inert_dy = idy / weight;
	   energy -= 10;
       surface->game_elements[ih]->energy -= 10;
       surface->game_elements[ih]->is_highlight = TRUE;
       if (surface->game_elements[ih]->energy <= 0) surface->game_elements[ih]->ai_mode = AI_MODE_ZEROENERGY_PREPARE;
	}

    // makes ship autofiring once for some ticks count
    if ((surface->tick_number & 0xF) == 0) {
      //surface->sounds->PlaySample(SOUND_SHOOT1);
      i = surface->LocateFreeGE(GM_AREA_FORE2);
      if (i >= 0) surface->game_elements[i] = new TGame_firepro(surface, (int)x-10, (int)y-26, FIREPRO_ROCKET);
      i = surface->LocateFreeGE(GM_AREA_FORE2);
      if (i >= 0) surface->game_elements[i] = new TGame_firepro(surface, (int)x+10, (int)y-26, FIREPRO_ROCKET);
    }
}


