//
// sound.cpp
// Classes for making music & sounds
//

#include "stdafx.h"
#include "resource.h"


TSoundSystem :: TSoundSystem ( TSurface * sf )
{
    int i;

    surface = sf;
    // Initialize FMOD sound system
    fmod_init_result = FSOUND_Init(11025 /* rate */, 32 /* soft channels */, 0 /* flags */);
    FSOUND_SetBufferSize(50);
    FSOUND_SetMixer(FSOUND_MIXER_MONO);
    // This is our stream - now only one stream for music
    stream = NULL;
    // Our samples pool
    for (i=0; i<MAX_SAMPLES_COUNT; i++) {
      sample[i] = NULL;
      sample_disposable[i] = TRUE;
    }
}

TSoundSystem :: ~TSoundSystem ()
{
    // Close stream
    FSOUND_Stream_Close(stream);
    // Samples must be freed automatically with this function
    FSOUND_Close();
}

// Place single sample into samples pool
//
void TSoundSystem :: LoadSample ( LPCTSTR folder, LPCTSTR fname, BOOL is_disposable )
{
    FSOUND_SAMPLE * fs;
    int    i;
    LPTSTR lps;
    TCHAR  s[256];
    char   sfname[1024];

    wsprintf(s, L"%s", fname);
    lps = wcschr(s, '_');
    if (lps != NULL) {
      *lps = 0;
      i = wcstol(s, NULL, 10);
      if ((i >= 0) && (i < MAX_SAMPLES_COUNT) && (sample[i] == NULL)) {
          sprintf(sfname, "%S%S%S", g_sCurrentDir, folder, fname);
          fs = FSOUND_Sample_Load(FSOUND_FREE, sfname, 0, 0, 0);
          if (fs != NULL) {
              sample[i] = fs;
              sample_disposable[i] = is_disposable;
          }
      }
    }
}

// Loads all samples from /sound/common folder - you must load them once when game starts
//
void TSoundSystem :: LoadCommonSamples ( void )
{
    HANDLE            ff;
    WIN32_FIND_DATA   fd;
    TCHAR             s[1024];

    wsprintf(s, L"%s\\sound\\common\\*.ogg", g_sCurrentDir);
	ff = FindFirstFile(s, &fd);
	if (ff != INVALID_HANDLE_VALUE) {
      // sounds must be named xxx_description.ogg where xxx is a number (e.g. 001_blast_big.ogg)
      LoadSample(L"\\sound\\common\\", fd.cFileName, FALSE/* is disposable */);
      while (FindNextFile(ff, &fd)) LoadSample(L"\\sound\\common\\", fd.cFileName, FALSE/* is disposable */);
	  FindClose(ff);
	}    
}

// Loads per-level samples ... you must describe them in sounds.txt file in /levelXXX folder
//
void TSoundSystem :: LoadLevelSamples ( int lev_num )
{
    HANDLE            ff;
    WIN32_FIND_DATA   fd;
    TCHAR             s[1024];

    wsprintf(s, L"%s\\sound\\level\\*.ogg", g_sCurrentDir);
	ff = FindFirstFile(s, &fd);
	if (ff != INVALID_HANDLE_VALUE) {
      // sounds must be named xxx_description.ogg where xxx is a number (e.g. 001_blast_big.ogg)
      LoadSample(L"\\sound\\level\\", fd.cFileName, TRUE/* is disposable */);
      while (FindNextFile(ff, &fd)) LoadSample(L"\\sound\\level\\", fd.cFileName, TRUE/* is disposable */);
	  FindClose(ff);
	}        
}

// Well ... Simple plays loaded sample 
//
void TSoundSystem :: PlaySample ( int num )
{
    if (fmod_init_result == FALSE) return;
    if (sample[num] == NULL) {
      TCHAR st[1024];
      wsprintf(st, L"PlaySample(): %u not loaded", num);
      surface->ConsoleOutput(st);
      return;
    }

    FSOUND_PlaySound(FSOUND_FREE, sample[num]);
}

// Initialize and start playing music stream
//
void TSoundSystem :: PlayMusic ( LPCTSTR s )
{
    char sfname[1024];

    if (stream != NULL) FSOUND_Stream_Close(stream);
    sprintf(sfname, "%S\\music\\%S", g_sCurrentDir, s);
    stream = FSOUND_Stream_Open(sfname, 0, 0, 0);
    if (stream != NULL) {
      FSOUND_Stream_Play(FSOUND_FREE, stream);
    }
}
