//
// surface.cpp
// Game surface class (a parent for all game objects)
// + Some helper classes
//

#include "stdafx.h"
#include "resource.h"


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        TSurface * game_surface = NULL;
        double GAME_SPEED_KOEFF = 1.0;


//////////////////////////////////////////////////////////////////////////
// Autocreator class
////////////////////////////////////////////////////////////////////////

//
// Autocreator will create some sort of objects automatically either with
// random or predefined location, e.g. asteroid storm
//
//

TAutocreator :: TAutocreator ( TSurface * sf )
{
	surface = sf;
    atype = 0;
}

void TAutocreator :: SetOn ( int __atype )
{
    if (__atype & AUTOCREATE_ASTERSWARM) {
      aswarm_bgr1_wait  = 1;
      aswarm_bgr2_wait  = 1;
      aswarm_fore1_wait = 1;
      aswarm_bgr1_idx   = 0;
      aswarm_bgr2_idx   = 0;
      aswarm_bgr1_off   = -1;
      aswarm_bgr2_off   = -100;
      aswarm_fore1_off  = -200;
      aswarm_all_off    = 0;
      atype = atype | AUTOCREATE_ASTERSWARM;
	}
    if (__atype & AUTOCREATE_STARDUST) {
      sdust_up2_wait = 1;
      atype = atype | AUTOCREATE_STARDUST;
    }
}

void TAutocreator :: SetOff ( int __atype )
{
    if (__atype & AUTOCREATE_ASTERSWARM) {
      aswarm_fore1_off = 1;   // timeout to switch off solid asteroids
      aswarm_bgr2_off  = 100; // timeout to switch off asteroid's background #2
      aswarm_bgr1_off  = 200; // --- / --- #1
      aswarm_all_off   = 200; // timeout to clear AUTOCREATE_ASTERSWARM bit
	}
    if (__atype & AUTOCREATE_STARDUST) {
      atype = atype & (!AUTOCREATE_STARDUST);
    }
}

void TAutocreator :: TimerTick ( void )
{
	int __x, __y, __imgnum, __esubtype;
    double __speed;
	int i;

	if (atype == 0) return;

    // Asteroids swarming ...
	if (atype & AUTOCREATE_ASTERSWARM) {
      if (aswarm_all_off > 1) aswarm_all_off--; else if (aswarm_all_off == 1) atype = atype - AUTOCREATE_ASTERSWARM;
      // far black asteroids - flying down with speed (2)
      if (aswarm_bgr1_off > 1) aswarm_bgr1_off--; else if (aswarm_bgr1_off < -1) aswarm_bgr1_off++;
      if ((aswarm_bgr1_off > 1) || (aswarm_bgr1_off == -1)) {
        aswarm_bgr1_wait -= (1.0 / GAME_SPEED_KOEFF);
		if (aswarm_bgr1_wait <= 0) {
          __x      = aster_swarm_array_bgr1[aswarm_bgr1_idx++];
          __imgnum = aster_swarm_array_bgr1[aswarm_bgr1_idx++];
          aswarm_bgr1_wait = aster_swarm_array_bgr1[aswarm_bgr1_idx++];
          if (aswarm_bgr1_wait < 0) { aswarm_bgr1_wait = -aswarm_bgr1_wait; aswarm_bgr1_idx = 0; }
          i = surface->LocateFreeGE(GM_AREA_BGR1);
          if (i >= 0) {
            __y = -surface->images[__imgnum]->dy[0];
            surface->game_elements[i] = new TGame_interior(surface, __x, __y, INTERIOR_ASTEROID_BGR, __imgnum);
            surface->game_elements[i]->speed = 2;
          }
        }
      }
      // more closer gray asteroids - flying down with speed (3)
      if (aswarm_bgr2_off > 1) aswarm_bgr2_off--; else if (aswarm_bgr2_off < -1) aswarm_bgr2_off++;
      if ((aswarm_bgr2_off > 1) || (aswarm_bgr2_off == -1)) {
        aswarm_bgr2_wait -= (1.0 / GAME_SPEED_KOEFF);
        if (aswarm_bgr2_wait <= 0) {
          __x      = aster_swarm_array_bgr2[aswarm_bgr2_idx++];
          __imgnum = aster_swarm_array_bgr2[aswarm_bgr2_idx++];
          aswarm_bgr2_wait = aster_swarm_array_bgr2[aswarm_bgr2_idx++];
          if (aswarm_bgr2_wait < 0) { aswarm_bgr2_wait = -aswarm_bgr2_wait; aswarm_bgr2_idx = 0; }
          i = surface->LocateFreeGE(GM_AREA_BGR2);
          if (i >= 0) {
            __y = -surface->images[__imgnum]->dy[0];
            surface->game_elements[i] = new TGame_interior(surface, __x, __y, INTERIOR_ASTEROID_BGR, __imgnum);
            surface->game_elements[i]->speed = 3;
          }
        }
      }        
      // enemy objects - solid asteroids flying down with random speed
      if (aswarm_fore1_off > 1) aswarm_fore1_off--; else if (aswarm_fore1_off < -1) aswarm_fore1_off++;
	  if ((aswarm_fore1_off > 1) || (aswarm_fore1_off == -1)) {
		aswarm_fore1_wait -= (1.0 / GAME_SPEED_KOEFF);
        if (aswarm_fore1_wait <= 0) {
          __x = MyRandom(5, screen_dx-5);
          __y = -50;
          __esubtype = MyRandom(0,3);
          __speed = MyRandom(14,18) / 4.0;
          aswarm_fore1_wait = MyRandom(15,25);
          i = surface->LocateFreeGE(GM_AREA_FORE1);
          if (i >= 0) {
            surface->game_elements[i] = new TGame_asteroid(surface, __x, __y, __esubtype);
            surface->game_elements[i]->speed = __speed;
		  }
        }
      }
    } // if AUTOCREATE_ASTERSWARM

    // Stardust 
	if (atype & AUTOCREATE_STARDUST) {
		sdust_up2_wait -= (1.0 / GAME_SPEED_KOEFF);
        if (sdust_up2_wait <= 0) {
          __x = MyRandom(-5, screen_dx-5);
          __y = -100;
          __imgnum = IMAGE_STARDUST;
          __speed = 1;
          sdust_up2_wait = MyRandom(60,70);
          i = surface->LocateFreeGE(GM_AREA_UP2);
          if (i >= 0) {
            surface->game_elements[i] = new TGame_interior(surface, __x, __y, INTERIOR_STARDUST, __imgnum);
            surface->game_elements[i]->speed = __speed;
		  }
        }
    }
}


//////////////////////////////////////////////////////////////
// TScriptParser class
////////////////////////////////////////////////////////////

//
// Class for parsing my simple level scripts
// 
//

// TScriptParser class - simple game level scripts processing
//
TScriptParser :: TScriptParser ( TSurface * sf )
{
    surface = sf;
    sdata.RemoveAll();
    curpos = NULL;
    current_command = SCRIPT_NOFILE;
	wait_ticks = 0;
}

// Script class destructor 
//
TScriptParser :: ~TScriptParser ()
{
    // TODO: something
}

// Loads a file into script-processor 
//
void TScriptParser :: LoadFile ( LPCTSTR fname )
{
    HANDLE f;
    CString s;
    BOOL   is_eof, was_data;

    sdata.RemoveAll();
    current_command = SCRIPT_NOFILE;
    curpos = NULL;

    s = g_sCurrentDir;
    s = s + L"\\";
    s = s + fname;
    f = CreateFile (s, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (f == INVALID_HANDLE_VALUE) {
      s = L"Unable to load: ";
      s = s + fname;
      surface->ConsoleOutput(s);
      return;
    }
    is_eof   = FALSE;
    was_data = FALSE;
    while (!is_eof) {
      is_eof = read_ansi_string(f, &s, 1000);
      s.TrimLeft();
      s.TrimRight();
      if ((s.GetLength() > 0) && (s.GetAt(0) != '#')) { was_data = TRUE; sdata.AddTail(s); }
    }
    if (was_data == TRUE) {
      // if was something then wait 1 tick (then further parsing)
      current_command = SCRIPT_WAIT;
      wait_ticks = 1; // ticks to wait
      curpos = sdata.GetHeadPosition();
    }
    CloseHandle(f);
}

// Process commands in script (strings in sdata)
//
void TScriptParser :: GetCommand ( void )
{
    CString s, scomm;
    int i,l,mode;
    TCHAR c;

    current_command = SCRIPT_NOCOMMAND;
	current_args[0] = L"";
	current_args[1] = L"";
	current_args[2] = L"";
	current_args[3] = L"";
	current_args[4] = L"";

    // reaches end of data
    if (curpos == NULL) {
      current_command = SCRIPT_ENDFILE;
      return;
    }

    s = sdata.GetNext(curpos);
    s.TrimLeft();
    s.TrimRight();
    // this must NOT happens but checking ... in case of fire
    if (s.GetLength() == 0) return; // no string to parse
    if (s.GetAt(0) == '#')  return; // string is a comment

    // split string to parts: scomm sarg[0] sarg[1] ...
    l = s.GetLength();
    i = 0;
    mode = 0;
    while (i < l) {
        c = s.GetAt(i);
        // end of string
        if ((c == 0x0D) || (c == 0x0A)) break; 
        // split parts by space or tab characters
        if ((c == 0x20) || (c == 0x09)) {
          if ((mode & 1) == 0) mode++;
          i++;
          continue;
        } else {
          if ((mode & 1) != 0) mode++;
        }
        // filling parts
        if (mode == 0) scomm = scomm + c;
        else
          if ((mode >= 2) && (mode <= 10) && ((mode & 1) == 0)) current_args[(mode>>1) - 1] = current_args[(mode>>1) - 1] + c;
        i++;
    }
    // now fill current_command args[0] args[1] ...
    scomm.TrimLeft();
    scomm.TrimRight();
    scomm.MakeUpper();
    if (scomm.GetLength() == 0) return;

    // WAIT command
    if (scomm == L"WAIT") {
      current_command = SCRIPT_WAIT;
      wait_ticks = wcstol(current_args[0], NULL, 10);
      return;
    }
    // INT command
    if (scomm == L"INT") {
      current_command = SCRIPT_INTERIOR;
      return;
    }
    // ASTERSWARM command
    if (scomm == L"ASTERSWARM") {
      current_command = SCRIPT_ASTERSWARM;
      return;
    }
    // STARDUST command
    if (scomm == L"STARDUST") {
      current_command = SCRIPT_STARDUST;
      return;
    }
    // GAMESPEED command
    if (scomm == L"GAMESPEED") {
      current_command = SCRIPT_GAMESPEED;
      return;
    }
    // RADIO command
    if (scomm == L"RADIO") {
      current_command = SCRIPT_RADIO;
      return;
    }
    // MUSIC command
    if (scomm == L"MUSIC") {
      current_command = SCRIPT_MUSIC;
      return;
    }
}

void TScriptParser :: TimerTick ( void )
{
    int   i;
	int	  __imgnum, __esubtype, __x, __y;
    double __speed;

    // no script - do nothing
    if (current_command == SCRIPT_NOFILE) return;

    // end of script - do nothing for now
    if (current_command == SCRIPT_ENDFILE) return;

    // if we a need to wait
    if (current_command == SCRIPT_WAIT) {
        if (wait_ticks <= 0) {
            current_command = SCRIPT_NOCOMMAND;
        } else {
            wait_ticks -= (1.0 / GAME_SPEED_KOEFF);
        }
        return;
    }

    // if we aren't waiting - parse further commands until reaches ENDFILE or WAIT
    while ((curpos != NULL) && (current_command != SCRIPT_ENDFILE) && (current_command != SCRIPT_WAIT)) {

      GetCommand();

      switch (current_command) {
        case SCRIPT_INTERIOR:
		  __esubtype = wcstol(current_args[0], NULL, 10);
		  __imgnum   = wcstol(current_args[1], NULL, 10);
		  __x        = wcstol(current_args[2], NULL, 10);
		  __y        = wcstol(current_args[3], NULL, 10);
		  __speed    = wcstod(current_args[4], NULL);
          i = surface->LocateFreeGE(GM_AREA_BGR2);
          if (i >= 0) {
			  surface->game_elements[i] = new TGame_interior(surface, __x, __y, __esubtype, __imgnum);
			  surface->game_elements[i]->speed = __speed;
		  }
          break;
		case SCRIPT_ASTERSWARM:
		  i = (int)wcstol(current_args[0], NULL, 10);
          if (i == 1) surface->autocreator->SetOn(AUTOCREATE_ASTERSWARM);
          if (i == 0) surface->autocreator->SetOff(AUTOCREATE_ASTERSWARM);
		  break;
		case SCRIPT_STARDUST:
		  i = (int)wcstol(current_args[0], NULL, 10);
          if (i == 1) surface->autocreator->SetOn(AUTOCREATE_STARDUST);
          if (i == 0) surface->autocreator->SetOff(AUTOCREATE_STARDUST);
		  break;
		case SCRIPT_GAMESPEED:
		  GAME_SPEED_KOEFF = wcstod(current_args[0], NULL);
		  break;
        case SCRIPT_RADIO:
		  i = (int)wcstol(current_args[0], NULL, 10);
          ((TGame_radio *)(surface->game_elements[GM_INDEX_RADIO]))->Talk(i, current_args[1], current_args[2]);
          break;
        case SCRIPT_MUSIC:
          surface->sounds->PlayMusic(current_args[0]);
          break;
      }

    }
}


///////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////
// Surface Class 
/////////////////////////////////////////////////////////////////

// Construct TSurface
//
TSurface :: TSurface ()
{
    int                 i;

    tick_number = 0;

    stylus_down = 0;
    stylus_last_x = 0;
    stylus_last_y = 0;

    game_mode = GM_MODE_UNKNOWN;

    // empty iface elements
    for (i=0; i<IFC_MAX_ELEMENTS_COUNT; i++) iface_elements[i] = NULL;
    // empty game elements
    for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) game_elements[i] = NULL;
	// empty console
	for (i=0; i<10; i++) console_buffer[i][0] = 0x00;
    // empty images (in case of fire)
    for (i=0; i<MAX_IMAGES_COUNT; i++) images[i] = NULL;

    // loads common images from gfx/iface folder, they are not disposable
    LoadImagesFromFolder(images, L"\\gfx\\iface\\", FALSE/* is disposable */);

    // Script parser
    script_parser = new TScriptParser(this);
    // Autocreator
    autocreator = new TAutocreator(this);
    // Main menu
    mainmenu = new TMainMenu(this);

    // Initialize soundsystem
    sounds = new TSoundSystem(this);
    // Load common samples
    sounds->LoadCommonSamples();

    // drawing must go on ...
    graph_out_enabled = TRUE;
    last_msec_count = GetTickCount();
    last_fps = 0;
}

// Destruct surface
//
TSurface :: ~TSurface()
{
    int i;

    // delete interface elements
    for (i=0; i<IFC_MAX_ELEMENTS_COUNT; i++) if (iface_elements[i] != NULL) {
        delete iface_elements[i];
        iface_elements[i] = NULL;
    }

    // delete game elements
    for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) if (game_elements[i] != NULL) {
        delete game_elements[i];
        game_elements[i] = NULL;
    }

    // delete images
    for (i=0; i<MAX_IMAGES_COUNT; i++) if (images[i] != NULL) { 
        delete images[i]; 
        images[i] = NULL; 
    }

    // delete script parser
    delete script_parser;

    // delete autocreator
    delete autocreator;

    // mainmenu
    delete mainmenu;

    // sounds
    delete sounds;
}


// Locate a free slot in game_elements[] array
// area - must be GM_AREA_XXX from defines
//
int TSurface :: LocateFreeGE ( WORD area )
{
    int i;
    for (i = LOBYTE(area); i<=HIBYTE(area); i++) if (game_elements[i] == NULL) {
      return i;
    }
    return -1;
}

// Draws full surface on screen
//
void TSurface :: DrawSurface ( void )
{
    int       i;

    if (graph_out_enabled == FALSE) return;

	// makes buffer blank
    memset(shadow_buffer, 0x00, screen_size*sizeof(WORD));

    // drawing game elements
    for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) 
        if (game_elements[i] != NULL) game_elements[i]->Draw();

	// drawing interface elements
    for (i=0; i<IFC_MAX_ELEMENTS_COUNT; i++) 
        if ((iface_elements[i] != NULL) && (iface_elements[i]->visible)) iface_elements[i]->Draw();

	// console output
	for (i=0; i<10; i++) 
        draw_string(shadow_buffer, 10, 300-(i*10), crect_fullscreen, console_buffer[i], images[IMAGE_FONTS],0, 6,8);

	// temporary info about some vars
	TCHAR s[1024];
	if (screen_mode == SCRMODE_GAPI_QVGA) wsprintf(s, L"MODE: GAPI QVGA");
	if (screen_mode == SCRMODE_RAW_QVGA) wsprintf(s, L"MODE: RAW QVGA");
	if (screen_mode == SCRMODE_RAW_VGA) wsprintf(s, L"MODE: RAW VGA (UPSCALE)");
	draw_string(shadow_buffer, 10,30, crect_fullscreen, s, images[IMAGE_FONTS],0, 6,8);
	if (stylus_down == 1) {
		wsprintf(s, L"MOUSE AT: %u %u", stylus_last_x, stylus_last_y);
		draw_string(shadow_buffer, 10,40, crect_fullscreen, s, images[IMAGE_FONTS],0, 6,8);
	}
    wsprintf(s, L"FPS: %u", last_fps);
	draw_string(shadow_buffer, screen_dx-55,30, crect_fullscreen, s, images[IMAGE_FONTS],0, 6,8);

    DrawShadowBuffer();
}

// Mouse down event
//
void TSurface::OnMouseDown( int mx, int my )
{
    int i;

    stylus_down = 1;
    stylus_last_x = mx;
    stylus_last_y = my;

    // if we are in main menu
    if (game_mode == GM_MODE_MAINMENU) {
      mainmenu->OnMouseDown(mx, my);
      return;
    }

    // searching (backwards) for interface elements 
    for (i=(IFC_MAX_ELEMENTS_COUNT-1); i>=0; i--) if ((iface_elements[i] != NULL) && (iface_elements[i]->visible)) {
        iface_elements[i]->OnMouseDown(mx, my);
    }
}

// Mouse up event
//
void TSurface::OnMouseUp( int mx, int my )
{
    int i;

    stylus_down = 0;
    stylus_last_x = mx;
    stylus_last_y = my;

    // if we are in main menu
    if (game_mode == GM_MODE_MAINMENU) {
      mainmenu->OnMouseUp(mx, my);
      return;
    }

    // searching (backwards) for interface elements 
    for (i=(IFC_MAX_ELEMENTS_COUNT-1); i>=0; i--) if ((iface_elements[i] != NULL) && (iface_elements[i]->visible)) {
        iface_elements[i]->OnMouseUp(mx, my);
    }
}

// Mouse move event
//
void TSurface::OnMouseMove( int mx, int my )
{
    stylus_down   = 1;
    stylus_last_x = mx;
    stylus_last_y = my;
}

// Timer ticks
//
void TSurface :: TimerTick ( void )
{
    int   i;
    DWORD d;

    tick_number++;

    // counting fps
    if ((tick_number & 0x3F) == 0) {
        d = GetTickCount();
        if (d == last_msec_count) last_fps = 99; else last_fps = (int)(64.0 * 1000.0 / ((double)d - (double)last_msec_count));
        last_msec_count = d;
    }

    // if we are in main menu mode
    //
    if (game_mode == GM_MODE_MAINMENU) {
        mainmenu->TimerTick();
        mainmenu->Draw();
        return;
    }

    // if we are playing level
    //
    if (game_mode == GM_MODE_NORMAL) {

      // if not paused - process game elements
      if (!is_paused) {
        // script processing
        script_parser->TimerTick();
        // autocreator processing
        autocreator->TimerTick();
        // searching for game elements 
        for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) if (game_elements[i] != NULL) {
          game_elements[i]->TimerTick();
          if (game_elements[i]->destroy_this == TRUE) {
            delete game_elements[i];
            game_elements[i] = NULL;
		  }
        }
      }

      // searching for interface elements - they will be processed even when paused
      for (i=0; i<IFC_MAX_ELEMENTS_COUNT; i++) if (iface_elements[i] != NULL) {
          iface_elements[i]->TimerTick();
          if (iface_elements[i]->destroy_this == TRUE) {
            delete iface_elements[i];
            iface_elements[i] = NULL;
          }
      }

      this->DrawSurface();
    }

}

//      
//
void TSurface :: ConsoleOutput ( LPCTSTR s )
{
	int	i;
	for (i=9; i>0; i--)
      wcscpy(console_buffer[i], console_buffer[i-1]);
	wcsncpy(console_buffer[0], s, 55);
}

//     () 
// :          
// (         -   )
//
BOOL TSurface :: CollisionCheck ( int x, int y, int dx, int dy, int * retindex, int * retdx, int * retdy )
{
    int i;
    int __dx, __dy;
    double r, __r, r2;
    TGameElement * gem;

    //    
    r = (dx>>1); if ((dy>>1) < r) r = (dy>>1);

    for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) if (game_elements[i] != NULL) {
      //      
      if (game_elements[i]->solid == FALSE) continue;
      gem = game_elements[i];
      __dx = images[gem->imgnum]->dx[(int)gem->sprnum];
      __dy = images[gem->imgnum]->dy[(int)gem->sprnum];

      //    
      __r = (__dx>>1); if ((__dy>>1) < __r) __r = (__dy>>1);
      //       
      r2 = (gem->x-x)*(gem->x-x) + (gem->y-y)*(gem->y-y);
      if (r2 > ((r+__r)*(r+__r))) continue;
      //  
      *retindex = i;
      //          
      if ((retdx != NULL) && (retdy != NULL)) {
		if (x < gem->x) *retdx = -20; else *retdx = 20;
		if (y < gem->y) *retdy = -20; else *retdy = 20;
        gem->inert_dx = -(*retdx) / (gem->weight);
        gem->inert_dy = -(*retdy) / (gem->weight);
	  }
      return TRUE;
    }
    return FALSE;
}




// Messages processing
//
void TSurface :: ProcessMessage ( int msg, WPARAM wParam, LPARAM lParam )
{
    int i;

    switch (msg) 
    {

    // delete interface element
    case WM_IF_DISPOSE: 
        for (i=0; i<IFC_MAX_ELEMENTS_COUNT; i++) if ((iface_elements[i] != NULL) && (iface_elements[i] == (TIfaceElement *)lParam)) {
          delete iface_elements[i];
          iface_elements[i] = NULL;
        }
        break;
    // delete game element
    case WM_GM_DISPOSE: 
        for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) if ((game_elements[i] != NULL) && (game_elements[i] == (TGameElement *)lParam)) {
          delete game_elements[i];
          game_elements[i] = NULL;
        }
        break;

    default:
        break;
    } // switch
}

// Keys processing
//
void TSurface :: ProcessGameKeys ( WPARAM vkey )
{
    switch( vkey )
    {
      case VK_LEFT:  break;
      case VK_RIGHT: break;
      case VK_UP:    break;
      case VK_DOWN:  break;
	  case 202: is_paused = !is_paused;
		        break;
    }
}

// ///////////////////////////////////////////////////////////////////////////////
// Game routines 
// /////////////////////////////////////////////////////////////////////////////

void TSurface :: StartGame ( void )
{
    StartLevel(1, FALSE);
}


void TSurface :: StartLevel ( int lev_num, BOOL is_continue )
{
    int i;

    if (is_continue == FALSE) {
      // default game variables
      Score = 0;
    } else {
      // TODO: load game variables from savegame file
    }

    // clears anything in game elements
    for (i=0; i<GM_MAX_ELEMENTS_COUNT; i++) if (game_elements[i] != NULL) { delete game_elements[i]; game_elements[i] = NULL; }
    // clears anything in interface elements
    for (i=0; i<IFC_MAX_ELEMENTS_COUNT; i++) if (iface_elements[i] != NULL) { delete iface_elements[i]; iface_elements[i] = NULL; }


    // delete all disposable images
    for (i=0; i<MAX_IMAGES_COUNT; i++) if ((images[i] != NULL) && (images[i]->is_disposable == TRUE)) {
      delete images[i];
      images[i] = NULL;
    }

    // delete all disposable sounds
    for (i=0; i<MAX_SAMPLES_COUNT; i++) if ((sounds->sample[i] != NULL) && (sounds->sample_disposable[i] == TRUE)) {
      FSOUND_Sample_Free(sounds->sample[i]);
      sounds->sample[i] = NULL;
    }

    // load this level images
    LoadImagesFromFolder(images, L"\\gfx\\level\\", TRUE/* is disposable */);

    // load this level sounds
    sounds->LoadLevelSamples(lev_num);


    // adding hud
    iface_elements[IF_INDEX_HUD] = new TIface_hud(this);

    // adding radio
    game_elements[GM_INDEX_RADIO] = new TGame_radio(this);

    // adding background stars
    game_elements[GM_INDEX_FSTARS] = new TGame_fstars(this, 40);

    // adding smoke class
    game_elements[GM_INDEX_SMOKE] = new TGame_smoke(this);

    // adding main ship
    game_elements[GM_INDEX_MAINSHIP] = new TGame_mainship(this);

    // starts parsing level script
    script_parser->LoadFile(L"level1.txt");

    game_mode = GM_MODE_NORMAL;
    screen_brightness_dir = -1; // fade in
}


////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////////////

// Creates new game surface instance and place some buttons
//
BOOL    InitSurface ( void )
{
    // create surface
    game_surface = new TSurface;

    // starts main menu
    game_surface->mainmenu->Load();
    game_surface->mainmenu->SetMenuLevel(1);
    game_surface->game_mode = GM_MODE_MAINMENU;

    return TRUE;
}

// Dispose all surface-related objects
//
void    DeinitSurface ( void )
{
    delete game_surface;
	game_surface = NULL;
}

