// Test1.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "resource.h"


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    // Global Variables:
    HINSTANCE			    g_hInst;				// The current instance
    HWND                    g_hMainWindow;          // Main window handle

	TCHAR                   g_sCurrentDir[2048];    // Current directory

    int                     g_wscreen_type;			// Display type, size and orientation related to mouse input
    int                     g_wscreen_dx;			//
    int                     g_wscreen_dy;			//
    int                     g_wscreen_orient;		//

	const TCHAR *           g_sWindowClass = L"WC_TEST1_CLASS1\0";
	const TCHAR *           g_sTitle = L"Test1\0";

    TCHAR                   g_sLastErrors[512];     // Last errors before program exit


////////////////////////////////////////////////////////////////////////////////////

// Add string to last errors buffer
//
void AddToGlobalErrors ( LPCTSTR s )
{
    wcsncat(g_sLastErrors, s, (sizeof(g_sLastErrors) - wcslen(g_sLastErrors) - 1));
}

// Adjust mouse coords to 240 x 320 starting in upper-left corner
//
void AdjustMouseParams ( LPARAM * lp )
{
	int	x, y;

	x = LOWORD(*lp);
    y = HIWORD(*lp);
    if (g_wscreen_type == SCRTYPE_VGA) { x = x >> 1; y = y >> 1; }
	*lp = (y << 16) | (x & 0xFFFF);
}

//  Processes messages for the main window.
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//TCHAR s[100];

    switch (message) 
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

        case WM_LBUTTONDOWN:
            if ((LOWORD(lParam) < 10) || (HIWORD(lParam) < 10)) { PostQuitMessage(0); return 0; }
			if (game_surface != NULL) {
			  AdjustMouseParams(&lParam);
              game_surface->OnMouseDown(LOWORD(lParam), HIWORD(lParam));
			}
            break;

        case WM_LBUTTONUP:
			if (game_surface != NULL) {
			  AdjustMouseParams(&lParam);
              game_surface->OnMouseUp(LOWORD(lParam), HIWORD(lParam));
			}
            break;

        case WM_MOUSEMOVE:
			if (game_surface != NULL) {
			  AdjustMouseParams(&lParam);
              game_surface->OnMouseMove(LOWORD(lParam), HIWORD(lParam));
			}
            break;

        case WM_KEYDOWN:
			if (game_surface != NULL) {
				//wsprintf(s, L"WM_KEYDOWN %u", wParam);
			    //game_surface->ConsoleOutput(s);
                game_surface->ProcessGameKeys(wParam);
			}
     		break;

        case WM_ACTIVATE:
			if (game_surface == NULL) break; // do nothing if no surface here
			if ((LOWORD(wParam) == WA_ACTIVE) || (LOWORD(wParam) == WA_CLICKACTIVE)) {
				SetForegroundWindow(g_hMainWindow);
				SetFocus(g_hMainWindow);
			    SHFullScreen(g_hMainWindow, SHFS_HIDETASKBAR);
			    SHFullScreen(g_hMainWindow, SHFS_HIDESIPBUTTON);
			    SHFullScreen(g_hMainWindow, SHFS_HIDESTARTICON);
				game_surface->graph_out_enabled = TRUE;
			}
			if (LOWORD(wParam) == WA_INACTIVE) {
				game_surface->graph_out_enabled = FALSE;
			    SHFullScreen(g_hMainWindow, SHFS_SHOWTASKBAR);
			    SHFullScreen(g_hMainWindow, SHFS_SHOWSIPBUTTON);
			    SHFullScreen(g_hMainWindow, SHFS_SHOWSTARTICON);
			}
     		break;

		case WM_SETTINGCHANGE:
            // TODO: rotation handling
			if (game_surface != NULL) game_surface->ConsoleOutput(L"WM_SETTINGCHANGE");
     		break;

        case WM_TIMER:
			if (game_surface != NULL) game_surface->TimerTick();
            return 0;

        case WM_PAINT:
            ValidateRect(g_hMainWindow, NULL);
            return 0;

        // custom messages to interface controls

        case WM_IF_DISPOSE:
        case WM_GM_DISPOSE:
            // dispose single game or interface element (pointed by lParam)
            if (game_surface != NULL) game_surface->ProcessMessage(message, wParam, lParam);
            break;

        default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}


// Shows last error in message box
//
void ShowLastError ( LPCTSTR sfunc ) 
{
    LPVOID lpMsgBuf;
	TCHAR  s[1024];

    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL, GetLastError(), 0, // Default language
        (LPTSTR) &lpMsgBuf, 0, NULL 
    );
	wcsncpy(s, (LPCTSTR)lpMsgBuf, 512);
	wcscat(s, L" (");
	wcsncat(s, sfunc, 128);
	wcscat(s, L")");
    // Free the buffer.
    LocalFree( lpMsgBuf );

    MessageBox( g_hMainWindow, s, g_sTitle, MB_OK | MB_ICONINFORMATION );
}

//  Registers the window class.
//
ATOM MyRegisterClass(HINSTANCE hInstance, LPCTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_TEST1));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

//  Saves instance handle and creates main window
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    int   wx;
    TCHAR s[1024];

    // If it is already running, then focus on the window
	g_hMainWindow = FindWindow(g_sWindowClass, g_sTitle);
	if (g_hMainWindow)
	{
		// set focus to foremost child window
		// The "| 0x01" is used to bring any owned windows to the foreground and
		// activate them.
		SetForegroundWindow((HWND)((ULONG) g_hMainWindow | 0x00000001));
		return 0;
	}

    // Register class
    if (!MyRegisterClass(g_hInst, g_sWindowClass)) {
      ShowLastError(L"Register Class");
      return FALSE;
    }

    g_wscreen_dx = GetSystemMetrics(SM_CXSCREEN);
    g_wscreen_dy = GetSystemMetrics(SM_CYSCREEN);
    g_wscreen_type = SCRTYPE_UNKNOWN;
    if ((g_wscreen_dx == 480) || (g_wscreen_dy == 480)) { g_wscreen_type = SCRTYPE_VGA; wx = 640; }
    if ((g_wscreen_dx == 240) || (g_wscreen_dy == 240)) { g_wscreen_type = SCRTYPE_QVGA; wx = 320; }
    if (g_wscreen_type == SCRTYPE_UNKNOWN) {
      // Error handling for unknown screen resolution ... 
      wsprintf(s, L"Unknown screen resolution: %u x %u", g_wscreen_dx, g_wscreen_dy);
      MessageBox(g_hMainWindow, s, g_sTitle, MB_OK);
      return FALSE;
    }

    // Get the current device settings
    DEVMODE sDevMode = {0};
    sDevMode.dmSize = sizeof(DEVMODE);
    sDevMode.dmFields = DM_DISPLAYORIENTATION;
    ChangeDisplaySettingsEx(NULL, &sDevMode, NULL, CDS_TEST, NULL);
    g_wscreen_orient = sDevMode.dmDisplayOrientation;

	// Create main window
	g_hMainWindow = CreateWindow(g_sWindowClass, g_sTitle, WS_VISIBLE, 0, 0, wx, wx, NULL, NULL, g_hInst, NULL);
    if (!g_hMainWindow) {
      ShowLastError(L"Create Window");
      return FALSE;
    }

	ShowWindow(g_hMainWindow, SW_SHOW);
	UpdateWindow(g_hMainWindow);
	SetForegroundWindow(g_hMainWindow);
	SetFocus(g_hMainWindow);

	// Hide taskbar, SIP button and Start icon
    SHFullScreen(g_hMainWindow, SHFS_HIDETASKBAR);
    SHFullScreen(g_hMainWindow, SHFS_HIDESIPBUTTON);
    SHFullScreen(g_hMainWindow, SHFS_HIDESTARTICON);

	return TRUE;
}


//
// Entry point
//
int WINAPI WinMain(	HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine,	int nCmdShow)
{
	MSG msg;

    // Store instance handle in our global variable
    g_hInst = hInstance;

	// Getting current directory
	GetModuleFileName(NULL, g_sCurrentDir, 2048);
    (*((wcsrchr(g_sCurrentDir,'\\'))+1))=0;

    // Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) return FALSE;
  
    // Init graphics engine
    if (!InitGfx()) {
      MessageBox(g_hMainWindow, g_sLastErrors, g_sTitle, MB_OK);
      return FALSE;
    }

    // New game surface
    if (!InitSurface()) {
      MessageBox(g_hMainWindow, g_sLastErrors, g_sTitle, MB_OK);
      return FALSE;
    }

    // Timer
    SetTimer(g_hMainWindow, 1, 40, NULL);

    // Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

    // Remove, unload and so on ...
    DeinitSurface();
    DeinitGfx();

	return 0;
}
